body {
  width: 650px;
  margin: 20px auto;
  font: 300 16px 'helvetica neue', helvetica, arial;
  text-shadow: 0 2px 2px #ccc;
}
h1 {
  font-size: 30px;
  font-weight: 300;
}
ol,li {
  list-style: none;
}
ol {
  -moz-box-shadow: 0 0 5px #ccc;
  -moz-border-radius: 5px;
  -webkit-box-shadow: 0 0 5px #ccc;
  -webkit-border-radius: 5px;
  box-shadow: 0 0 5px #ccc;
  border-radius: 5px;
  padding: 10px;
}
ol.sink-pass {
  background-color: lightgreen;
}
ol.sink-failure {
  background-color: lightpink;
}
li {
  margin: 3px 5px;
  padding: 5px;
  -webkit-border-radius: 5px;
  -moz-border-radius: 5px;
  border-radius: 5px;
  overflow: hidden;
  cursor: pointer;
}
ol > li {
  opacity: 0;
  background-color: lightblue;
  color: #000;
}
li span {
  color: #aaa;
  text-align: right;
  display: inline-block;
  float: right;
}
li.fail span {
  color: red;
}
li.pass span {
  color: green;
}
li.fail {
  background-color: #eed5d2;
}
li.pass {
  background-color: #bdfcc9;
}
li.pass, li.fail, li.mod {
  -webkit-transition: opacity .5s linear;
  opacity: 1;
}
li ul {
  height: 0;
  overflow: hidden;
  margin: 0;
  padding: 0;
  list-style: none;
  position: relative; /*early IE overflow bug*/
}
li ul.show {
  position: static; /*early IE overflow bug*/
}
li.pass ul.show {
  background-color: #8fffa3;
}
li.fail ul.show {
  background-color: #efada4;
}
li ul.show {
  height: auto;
  padding: 5px;
  -webkit-border-radius: 5px;
  -moz-border-radius: 5px;
  border-radius: 5px;
  margin: 5px;
}
li ul li {
  padding: 3px 6px;
  margin: 2px 0;
  list-style: none;
  font-size: 15px;
}
li b {
  display: block;
  margin-left: 40px;
}
li em.marker {
  display: block;
  position: absolute;
  right: 5px;
  top: 0;
}
